/*                                                                             
 * Copyright 2004,2005 The Apache Software Foundation.                         
 *                                                                             
 * Licensed under the Apache License, Version 2.0 (the "License");             
 * you may not use this file except in compliance with the License.            
 * You may obtain a copy of the License at                                     
 *                                                                             
 *      http://www.apache.org/licenses/LICENSE-2.0                             
 *                                                                             
 * Unless required by applicable law or agreed to in writing, software         
 * distributed under the License is distributed on an "AS IS" BASIS,           
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.    
 * See the License for the specific language governing permissions and         
 * limitations under the License.                                              
 */
package org.wso2.utils.transport;

import org.apache.axis2.AxisFault;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.SessionContext;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.transport.TransportListener;
import org.wso2.utils.NetworkUtils;
import org.wso2.utils.SessionContextUtil;

/**
 * 
 */
public abstract class AbstractTransportListener implements TransportListener {
    protected ConfigurationContext configurationContext;
    protected int port;
    protected TransportInDescription tInDescription;

    public void init(ConfigurationContext configContext,
                     TransportInDescription transportIn) throws AxisFault {
        this.configurationContext = configContext;
        this.port = Integer.parseInt(transportIn.getParameter("port").getValue().toString().trim());
        this.tInDescription = transportIn;
    }

    public void stop() throws AxisFault {
        // Do nothing. Servers will be stopped from elsewhere
    }

    public EndpointReference getEPR(String protocol,
                                    String serviceName,
                                    String ip) throws AxisFault {
        if(configurationContext == null){
            return null;
        }
        try {
            ip = NetworkUtils.getLocalHostname();
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
        String serviceContextPath = configurationContext.getServiceContextPath();
        if (serviceContextPath == null) {
            throw new AxisFault("couldn't find service path");
        }
        return new EndpointReference(protocol + "://" + ip + ":" + port
                                     + serviceContextPath +
                                     "/" + serviceName);
    }

    public EndpointReference getEPR(String protocol,
                                    String serviceName,
                                    String ip,
                                    String contextPath) throws AxisFault {
        if(configurationContext == null){
            return null;
        }
        try {
            ip = NetworkUtils.getLocalHostname();
        } catch (Exception e) {
            throw AxisFault.makeFault(e);
        }
        return new EndpointReference(protocol + "://" + ip + ":" + port
                                     + configurationContext.getServiceContextPath() +
                                     "/" + serviceName);
    }

    public SessionContext getSessionContext(MessageContext messageContext) {
        return SessionContextUtil.createSessionContext(messageContext);
    }

    public void destroy() {
        this.configurationContext = null;
    }
}
