/*
 * Copyright 2001-2004 The Apache Software Foundation.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.xkms2;

import java.security.Key;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.xml.security.algorithms.MessageDigestAlgorithm;
import org.apache.xml.security.c14n.Canonicalizer;
import org.apache.xml.security.exceptions.XMLSecurityException;
import org.apache.xml.security.signature.XMLSignature;
import org.apache.xml.security.transforms.Transforms;
import org.apache.xml.security.utils.IdResolver;
import org.w3c.dom.Element;
import org.wso2.xkms2.util.XKMSUtil;

public class RegisterRequest extends KRSSRequest {

    private PrototypeKeyBinding prototypeKeyBinding;

    private XMLSignature proofOfPossesion;

    private Key popKey;

    public void setProofOfPossesion(XMLSignature proofOfPossession) {
        this.proofOfPossesion = proofOfPossession;
    }

    public void setProofOfPocessionKey(Key popKey) {
        this.popKey = popKey;
    }

    public Key getProofOfPocessionKey() {
        return popKey;
    }

    public XMLSignature getProofOfPossession() {
        return proofOfPossesion;
    }

    public void setPrototypeKeyBinding(PrototypeKeyBinding prototypeKeyBinding) {
        this.prototypeKeyBinding = prototypeKeyBinding;
    }

    public PrototypeKeyBinding getPrototypeKeyBinding() {
        return prototypeKeyBinding;
    }

    public OMElement serialize(OMFactory factory) throws XKMSException {

        OMElement regRequestElement = factory.createOMElement(
                XKMS2Constants.Q_ELEM_REGISTER_REQUEST);
        super.serialize(factory, regRequestElement);
        
        regRequestElement.addChild(prototypeKeyBinding.serialize(factory));

        OMElement prototypeKeyBindingElement = regRequestElement
                .getFirstChildWithName(XKMS2Constants.Q_ELEM_PROTOTYPE_KEY_BINDING);
        String prototypeKeyBindingRefId = "#" + prototypeKeyBinding.getId();

        Authentication authentication = getAuthentication();

        if (authentication.getKeyBindingAuthentication() == null) {
            Key key = authentication.getKeyBindingAuthenticationKey();

            if (key != null) {
                XMLSignature keyBindingAuthSignature;
                try {

                    IdResolver.registerElementById(
                            (Element) prototypeKeyBindingElement,
                            prototypeKeyBinding.getId());

                    keyBindingAuthSignature = new XMLSignature(
                            ((Element) prototypeKeyBindingElement)
                                    .getOwnerDocument(), "",
                            XMLSignature.ALGO_ID_MAC_HMAC_SHA1,
                            Canonicalizer.ALGO_ID_C14N_EXCL_OMIT_COMMENTS);
                    Transforms transforms = new Transforms(
                            ((Element) prototypeKeyBindingElement)
                                    .getOwnerDocument());
                    transforms
                            .addTransform(Transforms.TRANSFORM_C14N_EXCL_OMIT_COMMENTS);
                    keyBindingAuthSignature.addDocument(
                            prototypeKeyBindingRefId, transforms,
                            MessageDigestAlgorithm.ALGO_ID_DIGEST_SHA1);
                    keyBindingAuthSignature.sign(key);
                    authentication
                            .setKeyBindingAuthentication(keyBindingAuthSignature);

                } catch (XMLSecurityException xse) {
                    throw new XKMSException(xse);
                }
            }
        }

        regRequestElement.addChild(authentication.serialize(factory));

        OMElement proofOfPossessionElement = factory.createOMElement(
                XKMS2Constants.Q_ELEM_PROOF_OF_POSSESSION, regRequestElement);
        if (proofOfPossesion == null) {
            try {
                IdResolver.registerElementById(
                        (Element) prototypeKeyBindingElement,
                        prototypeKeyBinding.getId());

                proofOfPossesion = new XMLSignature(
                        ((Element) prototypeKeyBindingElement)
                                .getOwnerDocument(), "",
                        XMLSignature.ALGO_ID_SIGNATURE_RSA_SHA1);
                Transforms transforms = new Transforms(
                        ((Element) proofOfPossessionElement).getOwnerDocument());
                transforms
                        .addTransform(Transforms.TRANSFORM_C14N_EXCL_OMIT_COMMENTS);
                proofOfPossesion.addDocument(prototypeKeyBindingRefId,
                        transforms, MessageDigestAlgorithm.ALGO_ID_DIGEST_SHA1);
                proofOfPossesion.sign(popKey);

            } catch (XMLSecurityException xse) {
                throw new XKMSException(xse);
            }
        }

        XKMSUtil.appendChild(proofOfPossesion.getElement(),
                proofOfPossessionElement);

        return regRequestElement;
    }
}
