/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.xkms2.core;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.context.ConfigurationContext;
import org.w3c.dom.Document;
import org.wso2.xkms2.*;
import org.wso2.xkms2.service.DefaultLocateService;
import org.wso2.xkms2.service.LocateService;

import java.util.Arrays;
import java.util.List;
import java.util.Iterator;

import sun.misc.Service;

/**
 * TODO Fix this for all scenarios
 */

public class LocateServiceExecutor implements XKMSServiceExecutor {

    private LocateService locateService;

    public OMElement execute(XKMSElement xkmsElement, OMFactory factory, Document document,
                             ProtocolExchange protocolExchange)
            throws XKMSException, AxisFault {

        LocateResult lresult = new LocateResult();

        try {

            LocateRequest lr =  (LocateRequest)xkmsElement;

            String id = lr.getId();
            lresult.setRequestId(id);
            lresult.setId(protocolExchange.generateRequestId());
            lresult.setServiceURI(lr.getServiceURI());

            QueryKeyBinding queryKeyBinding = lr.getQueryKeyBinding();
            List respondWithList = lr.getRespondWith();
            RespondWith[] respondWiths = null;
            if (respondWithList != null) {
                respondWiths = (RespondWith[]) respondWithList
                        .toArray(new RespondWith[respondWithList.size()]);
            }

            UnverifiedKeyBinding[] unverifiedKeyBindings =
                    locateService.locate(queryKeyBinding, respondWiths, document);
            // TODO supplying the states as required
            lresult.setResultMajor(ResultMajor.SUCCESS);

            if (unverifiedKeyBindings.length == 0) {
                lresult.setResultMinor(ResultMinor.NO_MATCH);
            }

            lresult.setUnverifiedKeyBindingList(Arrays.asList(unverifiedKeyBindings));

        } catch (XKMSException e) {
            /*TODO exceptions are not send rather state is maintain*/
            lresult.setResultMajor(ResultMajor.RECEIVER);
        }

        return lresult.serialize(factory);
    }

    public void init(ConfigurationContext configCtx) throws AxisFault {
        for (Iterator iterator = Service.providers(LocateService.class); iterator.hasNext();) {
            locateService = (LocateService) iterator.next();
            break;
        }
        try {
            String property = System.getProperty(LocateService.class.getName());
            if (property != null) {
                locateService = (LocateService) Class.forName(property).newInstance();
            }

            Parameter parameter =
                    configCtx.getAxisConfiguration().getParameter(LocateService.class.getName());
            if (parameter != null) {
                locateService = (LocateService) Class.forName((String) parameter.getValue())
                        .newInstance();
            }
        } catch (ClassNotFoundException e) {
            throw AxisFault.makeFault(e);
        } catch (IllegalAccessException e) {
            throw AxisFault.makeFault(e);
        } catch (InstantiationException e) {
            throw AxisFault.makeFault(e);
        }
        if (locateService == null) {
            locateService = new DefaultLocateService();
        }
    }

    public String getAssociatedElemenType() {
        return XKMS2Constants.ELE_LOCATE_REQUEST.getLocalPart();
    }

}
