/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.xkms2.core;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.om.OMFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.description.Parameter;
import org.apache.axis2.context.ConfigurationContext;
import org.w3c.dom.Document;
import org.wso2.xkms2.*;
import org.wso2.xkms2.service.DefaultValidateService;
import org.wso2.xkms2.service.ValidateService;

import java.util.List;
import java.util.Arrays;
import java.util.Iterator;

import sun.misc.Service;

/**
 * TODO fix this for all scenarios
 */

public class ValidateServiceExecutor implements XKMSServiceExecutor {

    ValidateService validateService;

    public OMElement execute(XKMSElement xkmsElement, OMFactory factory, Document document,
                             ProtocolExchange protocolExchange) throws XKMSException, AxisFault {


        ValidateResult vResult = new ValidateResult();

        ValidateRequest vr = (ValidateRequest) xkmsElement;

        vResult.setRequestId(vr.getId());
        vResult.setServiceURI(vr.getServiceURI());
        vResult.setId(protocolExchange.generateRequestId());

        QueryKeyBinding queryKeyBinding = vr.getQueryKeyBinding();
        List respondWithList = vr.getRespondWith();
        RespondWith[] respondWiths = null;
        if (respondWithList != null) {
            respondWiths = (RespondWith[]) respondWithList
                    .toArray(new RespondWith[respondWithList.size()]);
        }

        KeyBinding[] keyBindings =
                validateService.validate(queryKeyBinding, respondWiths, document);
        vResult.setResultMajor(ResultMajor.SUCCESS);

        if (keyBindings.length == 0) {
            vResult.setResultMinor(ResultMinor.NO_MATCH);
        }

        vResult.setKeyBindingList(Arrays.asList(keyBindings));

        return vResult.serialize(factory);
    }

    public void init(ConfigurationContext configCtx) throws AxisFault {
        for (Iterator iterator = Service.providers(ValidateService.class); iterator.hasNext();) {
            validateService = (ValidateService) iterator.next();
            break;
        }
        try {
            String property = System.getProperty(ValidateService.class.getName());
            if (property != null) {
                validateService = (ValidateService) Class.forName(property).newInstance();
            }

            Parameter parameter =
                    configCtx.getAxisConfiguration().getParameter(ValidateService.class.getName());
            if (parameter != null) {
                validateService = (ValidateService) Class.forName((String) parameter.getValue())
                        .newInstance();
            }
        } catch (ClassNotFoundException e) {
            throw AxisFault.makeFault(e);
        } catch (IllegalAccessException e) {
            throw AxisFault.makeFault(e);
        } catch (InstantiationException e) {
            throw AxisFault.makeFault(e);
        }

        if (validateService == null) {
            validateService = new DefaultValidateService();
        }
    }

    public String getAssociatedElemenType() {
        return XKMS2Constants.ELE_VALIDATE_REQUEST.getLocalPart();
    }
}
