/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.xkms2.core;

import org.apache.axiom.om.OMElement;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.AxisFault;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.receivers.AbstractInOutMessageReceiver;
import org.wso2.xkms2.XKMSException;
import org.wso2.xkms2.XKMS2Constants;
import org.wso2.xkms2.util.XKMSUtil;

import java.util.List;
import java.util.Arrays;
import java.util.Iterator;
/*
 *
 */

public class XKMSMessageReceiver extends AbstractInOutMessageReceiver {


    static {
        org.apache.xml.security.Init.init();
    }

    private ProtocolExchange protocolExchange;

    public XKMSMessageReceiver() {
        this.protocolExchange = new ProtocolExchange();
    }

    public void invokeBusinessLogic(MessageContext inMsgCtx, MessageContext outMsgCtx)
            throws AxisFault {

        try {

            initExecutors(inMsgCtx.getConfigurationContext());

            OMElement outXKMSElement = protocolExchange.exchangeServer(inMsgCtx);
            outXKMSElement = XKMSUtil.getOMElement(outXKMSElement);
            SOAPFactory fac = getSOAPFactory(inMsgCtx);
            SOAPEnvelope outEnvelop = fac.getDefaultEnvelope();
            outEnvelop.getBody().addChild(outXKMSElement);
            outMsgCtx.setEnvelope(outEnvelop);
        } catch (XKMSException e) {
            throw AxisFault.makeFault(e);
        }

    }


    private void initExecutors(ConfigurationContext configCtx) throws AxisFault {

        Object obj = configCtx.getProperty(XKMS2Constants.EXECUTOR_MAP);
        if (obj == null) {
            synchronized (this) {
                //TODO register all the builders. This can be done either this way or using Service Providers

                XKMSServiceExecutor[] xkmsServiceExecutors =
                        new XKMSServiceExecutor[]{
                                new LocateServiceExecutor(),
                                new ValidateServiceExecutor()};
                List xkmsSEArray = Arrays.asList(xkmsServiceExecutors);
                for (Iterator iterator = xkmsSEArray.iterator(); iterator.hasNext();) {
                    XKMSServiceExecutor xkmsServiceExecutor = (XKMSServiceExecutor) iterator.next();
                    xkmsServiceExecutor.init(configCtx);
                    XKMSServiceExecutorManager
                            .register(xkmsServiceExecutor.getAssociatedElemenType(),
                                      xkmsServiceExecutor);
                }
                configCtx.setProperty(XKMS2Constants.EXECUTOR_MAP, Boolean.TRUE);
            }
        }

    }
}
