/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.xkms2;

import junit.framework.TestCase;
import org.apache.axiom.om.OMAbstractFactory;
import org.apache.axiom.soap.SOAPEnvelope;
import org.apache.axiom.soap.SOAPFactory;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.context.MessageContext;
import org.apache.axis2.context.SessionContext;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.AxisServiceGroup;
import org.apache.axis2.description.TransportInDescription;
import org.apache.axis2.engine.AxisConfiguration;
import org.apache.axis2.transport.TransportListener;
import org.wso2.xkms2.core.XKMSMessageReceiver;
import org.wso2.xkms2.util.Messages;
/*
 * 
 */

public class LocateObjectTest extends TestCase {

    private MessageContext msgCtx;
    private TransportListener receiver;

    public void setUp() throws java.lang.Exception {
        org.apache.xml.security.Init.init();

        ConfigurationContext configCtx =
                ConfigurationContextFactory.createDefaultConfigurationContext();

        msgCtx = configCtx.createMessageContext();

        AxisConfiguration axisConfiguration = configCtx.getAxisConfiguration();

        TransportInDescription dummyTransport = new TransportInDescription("myTx");

        TransportListener tl = new TransportListener(){
            public void init(ConfigurationContext axisConf, TransportInDescription transprtIn){}
            public void start(){}

            public void stop(){}

            public EndpointReference getEPRForService(String serviceName, String ip) {
                return null;
            }

            public EndpointReference[] getEPRsForService(String serviceName, String ip) {
                return new EndpointReference[]{new EndpointReference("myTx://test.transport.org")};
            }
            public SessionContext getSessionContext(MessageContext messageContext) {
                return null;
            }
            public void destroy() {}
        };

        dummyTransport.setReceiver(tl);

        msgCtx.setTransportIn(dummyTransport);


        this.receiver = dummyTransport.getReceiver();
        this.receiver.start();

        AxisServiceGroup asg = new AxisServiceGroup(axisConfiguration);

        AxisService as = new AxisService("__TEST_XKMS_SERVICE_");
        axisConfiguration.addService(as);
        configCtx.createServiceGroupContext(asg);

        msgCtx.setAxisService(as);

    }

    public void testLocate() throws Exception {

        XKMSMessageReceiver xkmsMR = new XKMSMessageReceiver();

        SOAPFactory fac = OMAbstractFactory.getSOAP11Factory();

        SOAPEnvelope env = fac.getDefaultEnvelope();

        env.getBody().addChild(Messages.createDocSigLocateRequest());
        msgCtx.setEnvelope(env);

        xkmsMR.invokeBusinessLogic(msgCtx,new MessageContext());

    }

    public void tearDown() throws java.lang.Exception {
        receiver.stop();

    }
}
