/*
 * Copyright 2004,2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.wso2.xkms2.p2p;

import junit.framework.TestCase;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.context.ConfigurationContextFactory;
import org.apache.axis2.transport.http.SimpleHTTPServer;
import org.apache.axis2.transport.http.server.HttpUtils;
import org.apache.axis2.addressing.EndpointReference;
import org.apache.xml.security.keys.KeyInfo;
import org.apache.xml.security.keys.content.X509Data;
import org.wso2.xkms2.TestConstants;

import javax.xml.parsers.DocumentBuilderFactory;
import java.security.KeyStore;
import java.security.cert.X509Certificate;
import java.io.InputStream;
/*
 * 
 */

public class AbstractTestCase extends TestCase {

    private SimpleHTTPServer simpleHTTPServer;

    EndpointReference epr;

    protected void setUp() throws Exception {
        super.setUp();
        epr = new EndpointReference("http://" + HttpUtils.getIpAddress() + ":" +
                                    (TestConstants.TEST_PORT) +
                                    "/axis2/services/xkms");
        ConfigurationContext cc = ConfigurationContextFactory
                .createConfigurationContextFromFileSystem("target/test-repo", null);
        simpleHTTPServer = new SimpleHTTPServer(cc, TestConstants.TEST_PORT);
        simpleHTTPServer.start();
    }

    protected void tearDown() throws Exception {
        super.tearDown();
        simpleHTTPServer.stop();
    }

    protected KeyInfo getCertificate() throws Exception{
        KeyStore ks = KeyStore.getInstance("JKS");
        InputStream fis = Thread.currentThread().getContextClassLoader()
                .getResourceAsStream("./org/wso2/xkms/kiss/kiss-store.jks");

        ks.load(fis, "xkms-kiss".toCharArray());

        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);


        javax.xml.parsers.DocumentBuilder db = dbf.newDocumentBuilder();
        org.w3c.dom.Document doc = db.newDocument();
        KeyInfo ki = new KeyInfo(doc);

        doc.appendChild(ki.getElement());

        X509Certificate cert = (X509Certificate) ks.getCertificate("kiss");
        X509Data x509Data = new X509Data(doc);
        x509Data.addCertificate(cert);
        ki.add(x509Data);

        return ki;
    }
}
