/*
 * Copyright 2005-2007 WSO2, Inc. (http://wso2.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.wso2.wsas.transport.util;

import org.apache.axis2.AxisFault;
import org.apache.axis2.context.ConfigurationContext;
import org.apache.axis2.description.AxisOperation;
import org.apache.axis2.description.AxisService;
import org.apache.axis2.description.AxisServiceGroup;
import org.apache.axis2.engine.AxisConfiguration;
import org.wso2.wsas.ServerConstants;

import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Iterator;

public class ServiceHTMLProcessor {

    /**
     * Returns the HTML text for the list of services deployed.
     * This can be delegated to another Class as well
     * where it will handle more options of GET messages.
     *
     * @return Returns String.
     */
    public static String getServicesHTML(ConfigurationContext configContext) {
        String temp = "";
        Iterator serviceGroupsItr = configContext.getAxisConfiguration().getServiceGroups();
        boolean status = false;
        if (serviceGroupsItr.hasNext()) {
            status = true;
            temp += "<h2>" + "Deployed services" + "</h2>";
            while (serviceGroupsItr.hasNext()) {
                AxisServiceGroup axisServiceGroup = (AxisServiceGroup) serviceGroupsItr.next();
                if (!ServerConstants.ADMIN_SERVICE_GROUP.equals(axisServiceGroup.getServiceGroupName())) {
                    status = true;
                    for (Iterator serviceItr = axisServiceGroup.getServices();
                         serviceItr.hasNext();) {
                        AxisService axisService = (AxisService) serviceItr.next();
                        temp += "<h3><a href=\"" + axisService.getName() + "?info\">" +
                                axisService.getName() + "</a></h3>";
                    }
                }
            }
        }
        Hashtable erroneousServices =
                configContext.getAxisConfiguration().getFaultyServices();

        if ((erroneousServices != null) && !erroneousServices.isEmpty()) {
            temp += "<hr><h2><font color=\"blue\">Faulty Services</font></h2>";
            status = true;
            Enumeration faultyservices = erroneousServices.keys();
            while (faultyservices.hasMoreElements()) {
                String faultyserviceName = (String) faultyservices.nextElement();
                temp += "<h3><font color=\"blue\">" + faultyserviceName + "</font></h3>";
            }
        }

        if (!status) {
            temp = "<h2>There are no services deployed</h2>";
        }

        temp = "<html><head><title>Axis2: Services</title></head>" + "<body>" + temp
               + "</body></html>";

        return temp;
    }

    public static String printServiceHTML(String serviceName,
                                          ConfigurationContext configurationContext) {
        String temp = "";
        try {
            AxisConfiguration axisConfig = configurationContext.getAxisConfiguration();
            AxisService axisService = axisConfig.getService(serviceName);
            if (axisService != null) {
                if (!axisService.isActive()) {
                    temp += "<b>Service " + serviceName +
                            " is inactive. Cannot display service information.</b>";
                } else {
                    temp += "<h3>" + axisService.getName() + "</h3>";
                    temp += "<a href=\"" + axisService.getName() + "?wsdl\">wsdl</a> : ";
                    temp += "<a href=\"" + axisService.getName() + "?xsd\">schema</a> : ";
                    temp += "<a href=\"" + axisService.getName() + "?policy\">policy</a><br/>";
                    temp += "<i>Service Description :  " +
                            axisService.getDocumentation() + "</i><br/><br/>";
                    Iterator pubOpIter = axisService.getPublishedOperations().iterator();
                    if (pubOpIter.hasNext()) {
                        temp += "Published operations <ul>";
                        for (; pubOpIter.hasNext();) {
                            AxisOperation axisOperation = (AxisOperation) pubOpIter.next();
                            temp += "<li>" + axisOperation.getName().getLocalPart() + "</li>";
                        }
                        temp += "</ul>";
                    } else {
                        temp += "No operations specified for this service";
                    }
                }
            } else {
                temp += "<b>Service " + serviceName +
                        " not found. Cannot display service information.</b>";
            }
            temp = "<html><head><title>Service Information</title></head>" + "<body>" + temp
                   + "</body></html>";
        }
        catch (AxisFault axisFault) {
            temp = "<html><head><title>Service has a fualt</title></head>" + "<body>"
                   + "<hr><h2><font color=\"blue\">" + axisFault.getMessage() + "</font></h2></body></html>";
        }
        return temp;
    }

}
